import json

from ngSkinTools2.api import plugin


def get_related_skin_cluster(target):
    """
    Returns skinCluster if provided node name represents skinned mesh. Returns true for shapes that have
    skinCLuster in their deformation stack, or it's a skinCluster node itself.

    For invalid targets, returns None
    """

    return plugin.ngst2Layers(target, q=True, layerDataAttachTarget=True)


def unserialize_influences_from_json_data(info):
    from .influenceMapping import InfluenceInfo

    def as_influence_info(data):
        influence = InfluenceInfo()
        influence.pivot = data['pivot']
        influence.path = data.get('path', None)
        influence.name = data.get('name', None)
        influence.labelText = data['labelText']
        influence.labelSide = InfluenceInfo.SIDE_MAP[data['labelSide']]
        influence.logicalIndex = data['index']

        return influence

    if not info:
        return []

    return [as_influence_info(i) for i in info]


def list_influences(target):
    """
    :rtype: List[InfluenceInfo]
    """
    info = json.loads(plugin.ngst2Layers(target, q=True, influenceInfo=True))
    return unserialize_influences_from_json_data(info)


def is_slow_mode_skin_cluster(target):
    """
    returns true, if ngSkinTools chose to use slow ngSkinTools api for this target. Right now this only happens when skinCluster has non-transform
    nodes as influences (e.g. inverseMatrix node).
    """
    return plugin.ngst2Layers(target, q=True, skinClusterWriteMode=True) == "plug"
