from ngSkinTools2.api import plugin
from ngSkinTools2.api import layers as api_layers, Layer
from ngSkinTools2.api.target_info import list_influences
from ngSkinTools2.log import getLogger
from ngSkinTools2.python_compatibility import Object

log = getLogger("tools")


def assign_from_closest_joint(target, layer, influences=None):
    # type: (str, Layer, List[int]) -> None

    if influences is None:
        influences = [i.logicalIndex for i in list_influences(target)]

    if len(influences) == 0:
        # nothing to do?
        return

    plugin.ngst2tools(
        tool="closestJoint",
        target=target,
        layer=api_layers.as_layer_id(layer),
        influences=[int(i) for i in influences],
    )


def unify_weights(target, layer, overall_effect, single_cluster_mode):
    plugin.ngst2tools(
        tool="unifyWeights",
        target=target,
        layer=api_layers.as_layer_id(layer),
        overallEffect=overall_effect,
        singleClusterMode=single_cluster_mode,
    )


class FloodSettings(Object):
    def __init__(self):
        from ngSkinTools2.api import paint

        self.mode = paint.PaintMode.replace
        self.intensity = 1.0
        self.iterations = 1
        self.influences_limit = 0
        self.mirror = False
        self.fixed_influences_per_vertex = False
        self.distribute_to_other_influences = False


def flood_weights(layer, influence=None, settings=None):
    """
    :type target: str
    :type layer: Layer
    :type settings: FloodSettings
    """

    if settings is None:
        settings = FloodSettings()  # just use default settings

    plugin.ngst2tools(
        tool="floodWeights",
        target=layer.mesh,
        influence=influence,
        layer=api_layers.as_layer_id(layer),
        mode=settings.mode,
        intensity=settings.intensity,
        iterations=int(settings.iterations),
        influencesLimit=int(settings.influences_limit),
        mirror=bool(settings.mirror),
        distributeRemovedWeight=settings.distribute_to_other_influences,
        fixedInfluencesPerVertex=bool(settings.fixed_influences_per_vertex),
    )
