from ngSkinTools2 import api, signal, operations
from ngSkinTools2.observableValue import ObservableValue
from ngSkinTools2.operations import layers
from ngSkinTools2.python_compatibility import Object
from ngSkinTools2.ui import dialogs
from ngSkinTools2.ui.session import Session


def __create_tool_action__(parent, session, action_name, action_tooltip, exec_handler):
    """
    :type session: Session
    """

    from ngSkinTools2.ui import actions

    def execute():
        if not session.active() or session.state.currentLayer.layer is None:
            return

        exec_handler()

    result = actions.define_action(parent, action_name, callback=execute)
    result.setToolTip(action_tooltip)

    @signal.on(session.events.targetChanged, session.events.currentLayerChanged)
    def update_state():
        result.setEnabled(session.state.layersAvailable and session.state.currentLayer.layer is not None)

    return result


class ClosestJointOptions(Object):
    create_new_layer = ObservableValue(False)
    all_influences = ObservableValue(True)


def create_action__from_closest_joint(parent, session):
    options = ClosestJointOptions()

    def exec_handler():
        layer = session.state.currentLayer.layer
        influences = None
        if not options.all_influences():
            influences = list(session.context.get_selected_influences())
            if not influences:
                dialogs.info("Select one or more influences in Influences list")
                return

        if options.create_new_layer():
            layer = layers.addLayer()

        api.assign_from_closest_joint(
            session.state.selectedSkinCluster,
            layer,
            influences=influences,
        )
        session.events.currentLayerChanged.emitIfChanged()
        session.events.influencesListUpdated.emit()

        if layer.paint_target is None:
            used_influences = layer.get_used_influences()
            if used_influences:
                layer.paint_target = min(used_influences)

    return (
        __create_tool_action__(
            parent,
            session,
            action_name=u"Assign From Closest Joint",
            action_tooltip="Assign 1.0 weight for closest influence per each vertex in selected layer",
            exec_handler=exec_handler,
        ),
        options,
    )


class UnifyWeightsOptions(Object):
    overall_effect = ObservableValue(1.0)
    single_cluster_mode = ObservableValue(False)


def create_action__unify_weights(parent, session):
    options = UnifyWeightsOptions()

    def exec_handler():
        api.unify_weights(
            session.state.selectedSkinCluster,
            session.state.currentLayer.layer,
            overall_effect=options.overall_effect(),
            single_cluster_mode=options.single_cluster_mode(),
        )

    return (
        __create_tool_action__(
            parent,
            session,
            action_name=u"Unify Weights",
            action_tooltip="For selected vertices, make verts the same for all verts",
            exec_handler=exec_handler,
        ),
        options,
    )
